/*************************************/
/* Fichier: matj.c                   */
/* Version: 2.0                      */
/* Date:   July 2011                 */
/* Author: Didier Pinchon            */
/*************************************/
#include "rotation.h"

MatL *read_lmatJ(int lmax)
{
   int l,k,k1,i,j,lmaxmax;
   double x;
   char com[10];
   MatL *matj;
   FILE *fin;
   
   if ((fin = fopen("numMatJ.dat","r")) == NULL) {
     fprintf(stderr,"Cannot open input file numMatJ.dat.\n");
   }
   
   matj = matl_calloc(lmax);
   
   fscanf(fin,"%s %d",com,&lmaxmax);
   if (lmax>lmaxmax) {
      fprintf(stderr,"lmax is too small in numMatJ.dat.\n");
   }
   
   gsl_matrix_set(matj->lmat[0],0,0, 1.0);
   gsl_matrix_set(matj->lmat[1],0,1, -1.0);
   gsl_matrix_set(matj->lmat[1],1,0, -1.0);
   gsl_matrix_set(matj->lmat[1],2,2, 1.0);
   
   for (l=2;l<=lmax;l++) {
   fscanf(fin,"%s %d",com,&lmaxmax);
      k = l/2;
      k1 = k;
      for (i=0;i<k;i++) {
         for (j=0;j<i;j++) {
	    fscanf(fin,"%lf",&x);
            gsl_matrix_set(matj->lmat[l],2*i+1,2*j+1,x);
	    gsl_matrix_set(matj->lmat[l],2*j+1,2*i+1,x);
	 }
	 fscanf(fin,"%lf",&x);
	 gsl_matrix_set(matj->lmat[l],2*i+1,2*i+1,x);
      }
      if (l!=2*k1) k++;
      for (i=0;i<k;i++) {
         for (j=0;j<k;j++) {
	    fscanf(fin,"%lf",&x);
            gsl_matrix_set(matj->lmat[l],2*k1+2*j+1,2*i,x);
	    gsl_matrix_set(matj->lmat[l],2*i,2*k1+2*j+1,x);
         }
       }
       if (l==2*k1) {k++;} else {k1++;}
        for (i=0;i<k;i++) {
         for (j=0;j<i;j++) {
	    fscanf(fin,"%lf",&x);
            gsl_matrix_set(matj->lmat[l],2*k1+2*i,2*k1+2*j,x);
	    gsl_matrix_set(matj->lmat[l],2*k1+2*j,2*k1+2*i,x);
	 }
	 fscanf(fin,"%lf",&x);
	 gsl_matrix_set(matj->lmat[l],2*k1+2*i,2*k1+2*i,x);
       } 
   }
   fclose(fin);
   return matj;
}

gsl_matrix *read_matJ(int l)
{
   int k,k1,i,j, lval;
   double x;
   char com[10] ,fname[20];
   gsl_matrix *matj;
   FILE *fin;

   matj = gsl_matrix_calloc(2*l+1,2*l+1); 

   if (l==0) {
     gsl_matrix_set(matj,0,0, 1.0);
     return matj;
   }
   if (l==1) {
     gsl_matrix_set(matj,0,1, -1.0);
     gsl_matrix_set(matj,1,0, -1.0);
     gsl_matrix_set(matj,2,2, 1.0);
     return matj;
   }
   
   sprintf(fname,"NumMatJ/numMatJ-%d.dat",l);
 
   if ((fin = fopen(fname,"r")) == NULL) {
     fprintf(stderr,"Cannot open input file %s.\n",fname);
   }
   
   fscanf(fin,"%s %d",com,&lval);
   if (lval!=l) {
      fprintf(stderr,"bad value of l in %.\n",fname);
   }

   
   
   k = l/2;
   k1 = k;
   for (i=0;i<k;i++) {
         for (j=0;j<i;j++) {
	    fscanf(fin,"%lf",&x);
            gsl_matrix_set(matj,2*i+1,2*j+1,x);
	    gsl_matrix_set(matj,2*j+1,2*i+1,x);
	 }
	 fscanf(fin,"%lf",&x);
	 gsl_matrix_set(matj,2*i+1,2*i+1,x);
   }
   if (l!=2*k1) k++;
   for (i=0;i<k;i++) {
      for (j=0;j<k;j++) {
	    fscanf(fin,"%lf",&x);
            gsl_matrix_set(matj,2*k1+2*j+1,2*i,x);
	    gsl_matrix_set(matj,2*i,2*k1+2*j+1,x);
         }
    }
    if (l==2*k1) {k++;} else {k1++;}
    for (i=0;i<k;i++) {
         for (j=0;j<i;j++) {
	    fscanf(fin,"%lf",&x);
            gsl_matrix_set(matj,2*k1+2*i,2*k1+2*j,x);
	    gsl_matrix_set(matj,2*k1+2*j,2*k1+2*i,x);
	 }
	 fscanf(fin,"%lf",&x);
	 gsl_matrix_set(matj,2*k1+2*i,2*k1+2*i,x);
   } 
   fclose(fin);
   return matj;
}

