/// @file
////////////////////////////////////////////////////////////////////////////////
///
/// Copyright (C) 2009      University of Toronto
///
////////////////////////////////////////////////////////////////////////////////
///
///  module     : condition number optimization
///
///  author     : lessig@dgp.toronto.edu
///
///  project    : Spherical Harmonics rotation
///
///  description: computation of "energy" for optimization
///
///  license    : See README for details.
///
////////////////////////////////////////////////////////////////////////////////

#ifndef _ENERGY_H_
#define _ENERGY_H_

// includes, Spherical Harmonics
#include <sh_evaluator.h>

//! Compute energy
class Energy {

public:

  //! Constructor
  Energy();
  
  //! Destructor
  ~Energy();

public:

  //! Initialization
  void init( const unsigned int num_samples, const unsigned int band, 
             const double eps_theta, const double eps_phi);

  //! Compute energy and gradient at \a lambdas
  void compute( const LambdaVec& lambdas, double& energy, Vec& gradient);

public:

  inline const unsigned int& getShBand() const {
    return sh_band;
  }

private:

  //! Transpose kernel matrix to match Fortran format used in Lapack
  void transposeQ();

  //! Compute kernel matrix for set of sampling \a lambdas
  void computeKernelMatrix( const LambdaVec& lambdas);

  //! Compute condition number 
  //! @return condition number
  double  computeCondNum( const LambdaVec& lambdas);

private:

  unsigned int sh_band;
  unsigned int sh_band_dim;
  unsigned int num_samples;
  double  fd_eps_theta;
  double  fd_eps_phi;

  Vec  svals;

  double*  q_data;
  double*  q_trans;

  double*    ws;
  int   ws_size;

  ShEvaluator<double>*        sh_eval;
  std::vector<double>         sh_vals;

private:

  //! Constructor, copy 
  //! @note Not implemeted
  Energy( Energy&);

  //! Assignment operator
  //! @note Not implemented
  Energy& operator=( Energy&);

};

#endif // _ENERGY_H_
