// L-BFGS wrapper code, written by Denis Zorin


#ifndef __BFGS_WRAP_H__
#define __BFGS_WRAP_H__

// #include "compat.h"
#include <cmath>
#include <cassert>

class Domain { 

public:

  typedef enum  { None = 0, LowerOnly = 1, Both = 2, UpperOnly = 3} BoundType;
  int size() { return dim; }
  double* lower() { return lo;}
  double* upper() { return up;}
  int*    boundtype() { return bnd;}

  void setBound(int i, double l, double u, BoundType b ) { 
    assert(i < dim);
    up[i] = u; lo[i] = l; bnd[i] = b;
  }

  Domain(int n) :dim(n) { 
     lo = new double[n]; up = new double[n]; bnd = new int[n];
     for(int i=0;i<n;i++)
       bnd[i] = None;
  }
  ~Domain() { 
    delete[] lo; delete[] up; delete[] bnd;
  }
private:
  int dim;
  double* lo;
  double* up;
  int*    bnd;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class BFGSoptimizer {
 public: 
  typedef enum { 
    EvaluationRequest, // before next call for next_iteration recompute f and grad
    Abnormal,     
    DataError,   // error in the input data
    Converged,    // convergence criterion was met
    IterationCompleted, // iteration completed (no recalculation of f, g required)
    Unrecognized // could not interpret return from Fortran
  } Results;

  // the first argument is the dimension of input, the second is the
  // intial point in the domain, 
  BFGSoptimizer( int n, double* x0, Domain* ds = 0, int m = 0);

  ~BFGSoptimizer() { 
    delete[] wa;
    delete[] iwa;
    if(mydomain) delete[] domain;
  }

  void setAccuracyFactor(double d)   { factr = d; }
  void setGradientAccuracy(double d) { pgtol = d; }
  void setPrintLevel(int i) { iprint = i; }
  
  Results  nextIteration(double f, double* grad ); 
  
 private:
  int dim;
  int matdim;
  double* x;
  bool mydomain;
  Domain* domain;
  double f;
  double* g;
  double factr;
  double pgtol;
  
  char  task[60];
  int iprint;
  
 // internal workspace for setlub
  double* wa;
  int* iwa;
  char csave[60];
  int  lsave[4];
  int  isave[44];
  double dsave[29]; 
};

#endif /* __BFGS_WRAP_H__*/






