/// @file
////////////////////////////////////////////////////////////////////////////////
///
/// Copyright (C) 2009      University of Toronto
///
////////////////////////////////////////////////////////////////////////////////
///
///  module     : condition number optimization
///
///  author     : lessig@dgp.toronto.edu
///
///  project    : Spherical Harmonics rotation
///
///  description: File I/O for data transfer with matlab
///
///  license    : See README for details.
///
////////////////////////////////////////////////////////////////////////////////

#ifndef _FILE_IO_H_
#define _FILE_IO_H_

// includes, file
#include <config.h>

namespace FileIO {

  //! Read Matlab ascii file containing the set of sampling { \lambda_i } and
  //! return the data in a C compatible format  
  //! @param path  path to input file 'lambda_in.dat'
  void
  readMatlabLambdaFile( const std::string& path, unsigned int& sh_band,
                        unsigned int& potential,
                        double& eps_theta, double& eps_phi, LambdaVec& lambdas);

  //! Write Matlab ascii file
  //! @param path  path to output file 'lambda_out.dat', 'energy_out.dat'
  void
  writeMatlabLambdaFile( const std::string& path, const double energy,
                         const LambdaVec& lambdas);

};

#endif // _FILE_IO_H_

