#include <cfloat>   // define DBL_MIN and DBL_MAX; might need a different
                      // lib on your system

#include <cstdio>
#include <iostream>
#include <cmath>
#include <cassert>
#include <vector>
#include <algorithm>

#include "BFGSwrap.h"

// typedefs
typedef std::vector<double>  Vec;

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
template<class T>
void
ComputeEnergy( const T& var, T& energy, T& gradient) {

  energy = cos(var);

  gradient = -1.0 * sin(var);
}

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
int
main( int /*argc*/, char** /*argv*/) {

  unsigned int num_variables = 1;

  // create a domain without constraints
  Domain dom( num_variables);
  for( unsigned int i=0; i < num_variables; i++) {
    dom.setBound( i, DBL_MIN, DBL_MAX, Domain::None);
  }

  // initialize variables
  double omega = 0.1;

  // initialize optimizer
  BFGSoptimizer opt( num_variables, &omega, &dom, 15);
  opt.setPrintLevel(1); 
  
  double energy = 0.0;
  double gradient = 0.0;
  ComputeEnergy( omega, energy, gradient);
 
  // unsigned int i = 0;
  // double lambda = 0.1;
  // unsigned int N = train[i].size();

  BFGSoptimizer::Results res;
  do { 

    // it is ok to call it first time without computing f and g: 
    // it will return EvaluationRequest  
    res  = opt.nextIteration( energy, &gradient);

    if( res == BFGSoptimizer::EvaluationRequest ) { 
      ComputeEnergy( omega, energy, gradient);
    } 
  } while( ( res == BFGSoptimizer::IterationCompleted )|| 
           ( res == BFGSoptimizer::EvaluationRequest));

  std::cerr << "x = " << omega 
            << " :: energy = " << energy 
            << " :: gradient = " << gradient << std::endl;
  
  return 0;
}

