/// @file
////////////////////////////////////////////////////////////////////////////////
///
/// Copyright (C) 2009      University of Toronto
///
////////////////////////////////////////////////////////////////////////////////
///
///  module     : condition number optimization
///
///  author     : lessig@dgp.toronto.edu
///
///  project    : Spherical Harmonics rotation
///
///  description: main file
///
///  license    : See README for details.
///
////////////////////////////////////////////////////////////////////////////////

// includes, system
#include <cfloat>  
#include <iostream>
#include <cmath>
#include <vector>
#include <iomanip>

// includes, project
#include <config.h>
#include <file_io.h>
#include <energy.h>
#include <optimizer.h>

////////////////////////////////////////////////////////////////////////////////
//! Extract base path for project
////////////////////////////////////////////////////////////////////////////////
void
getBasePath( std::string& path) {

  std::size_t pos = path.rfind( "/");
  pos = path.rfind( "/", pos-1);
  pos = path.rfind( "/", pos-1);
  if( pos != std::string::npos) {
    path = path.substr( 0, pos);
  }
  path += "/data";
}

////////////////////////////////////////////////////////////////////////////////
//! Program main
////////////////////////////////////////////////////////////////////////////////
int
main( int argc, char** argv) {

  std::string path( argv[0]);
  getBasePath( path);

  // additional directory for instance
  std::string path_instance( "");
  if( argc > 1) {
    //path += "/";
    path = argv[1];
  }
  std::cout << "Path = " << path << "\n";

  // check if a maximum optimization time has been specified
  unsigned int opt_time_max = 0 - 1;
  if( argc > 2) {
    opt_time_max = atoi( argv[2]);
  }
  std::cout << "\nOptimzation time = " << opt_time_max << " seconds." << "\n";

  // parameter setup
  
  Optimizer optimizer;
  optimizer.init( path, opt_time_max);
  optimizer.run();
  optimizer.finish( path);

  return 0;
}

