/*************************************/
/* Fichier: rotation.h               */
/* Version: 2.0                      */
/* Date:   July 2011                 */
/* Author: Didier Pinchon            */
/*************************************/
#ifndef ROTATION_H
#define ROTATION_H

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <gsl_vector.h>
#include <gsl_matrix.h>
#include <gsl_blas.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct S_VECL {
   int lmax;
   gsl_vector **lvec;
} VecL;

typedef struct S_MATL {
   int lmax;
   gsl_matrix **lmat;
} MatL;

/* alloc.c */
extern VecL *vecl_calloc(int lmax);
extern void  vecl_free(VecL *vecl);
extern MatL *matl_calloc(int lmax);
extern void  matl_free(MatL *matl);

/* matj.c */
extern  MatL       *read_lmatJ(int lmax);
extern  gsl_matrix *read_matJ(int l);

/* rotation_matrix */
extern int   matl_matl_product(const MatL *matl1, const MatL *matl2, MatL *matres);
extern int   matl_rotation(int lmax,double alpha, double beta, double gamma,MatL *res,const MatL *matj);
extern int   matrix_rotation(int l,double alpha, double beta, double gamma,gsl_matrix *res,const gsl_matrix *matj);

/* rotation_vector.c */
extern int matl_vecl_product(const MatL *matl, const VecL *vecl, VecL *res);
extern int vecl_rotation(int lmax,double alpha, double beta, double gamma, const VecL *x, VecL *res, const MatL *matj);
extern int vector_rotation(int l,double alpha, double beta, double gamma,const gsl_vector *x,gsl_vector *res,const gsl_matrix *matj);

#ifdef __cplusplus
}
#endif

#endif /* ROTATION_H */
