# rotation.mpl 
# MEPOM - Didier Pinchon
# Version 1.0 , 1 September 2006
# rotation matrices for real spherical harmonics

read("spherical.mpl"):
read("gaunt.mpl"):
with(linalg):

# real spherical harmonics with variables x,y,z
Sxyz := proc(l,m)
  local leq,res;
  leq := {cos(theta)=z/r,sin(theta)=sqrt(x^2+y^2)/r,cos(phi)=x/sqrt(x^2+y^2),
   sin(phi)=y/sqrt(x^2+y^2)};
  res := simplify(r^l*subs(leq,expand(SS(l,m))));
  res := simplify(subs(r=sqrt(x^2+y^2+z^2),res));

end:

Sbasis := proc(l) [seq(Sxyz(l,m),m=-l..l)]  end:

# complex spherical harmonics with variables x,y,z
Yxyz := proc(l,m)
  local leq,res;
  leq := {cos(theta)=z/r,sin(theta)=sqrt(x^2+y^2)/r,cos(phi)=x/sqrt(x^2+y^2),
   sin(phi)=y/sqrt(x^2+y^2)};
  res :=simplify(r^l*subs(leq,expand(convert(YY(l,m),trig))));
  res := simplify(subs(r=sqrt(x^2+y^2+z^2),res));
end:

Ybasis := proc(l) [seq(Yxyz(l,m),m=-l..l)] end:

setCoeffs := proc(f,lv)
   local g,lc;
   # Maple "coeffs" does not work here
   lc := {seq(coeff(f,lv[1],i),i = 0..degree(f,lv[1]))}; 
   if nops(lv)=1 then RETURN(lc); fi;
   {seq(op(setCoeffs(g,lv[2..-1])), g = lc)};
end:

linCoeffs := proc(f,basis)
  local g,leq,i,sol,dim;
  dim := nops(basis);
  g := add(c[i]*basis[i],i=1..dim)- expand(f);
  leq:= setCoeffs(g,[x,y,z]);
  sol := solve(leq,{seq(c[i],i=1..dim)});
  [seq(combine(subs(sol,c[i])),i=1..dim)];
end:

# Matrix of a linear operator in a space given a base
opMatrix := proc(opf,basis)
   local mat,i,j,dim,lco;
   dim := nops(basis);
   mat := matrix(dim,dim,0);
   for j to dim do
     lco := linCoeffs(opf(basis[j]), basis);
     for i to dim do
       mat[i,j] := simplify(lco[i]);
     od;
   od;
   mat;
end:

# Matrix of a linear operator between two spaces given two bases fam and basis
opMatrix2 := proc(opf,fam,basis)
   local mat,i,j,dim,dim2,lco;
   dim := nops(basis);
   dim2 := nops(fam);
   mat := matrix(dim,dim2,0);
   for j to dim2 do
     lco := linCoeffs(opf(fam[j]), basis);
     for i to dim do
       mat[i,j] := simplify(lco[i]);
     od;
   od;
   mat;
end:

# Change of base matrix
changeBase := proc(bas1,bas2) opMatrix2(a -> a, bas1,bas2); end:

# Permutation matrix
permutationMat := proc(ll)
  local dim,mat,i;
  dim := nops(ll);
  mat := matrix(dim,dim,0);
  for i to dim do
    mat[i,ll[i]]:=1;
  od;
  mat;
end:
    
# The elementary rotations
rotXY := proc(f,alpha) subs({x=cos(alpha)*x+sin(alpha)*y,y=-sin(alpha)*x+cos(alpha)*y},f) end:
rotXZ := proc(f,alpha) subs({x=cos(alpha)*x+sin(alpha)*z,z=-sin(alpha)*x+cos(alpha)*z},f) end:
rotYZ := proc(f,alpha) subs({y=cos(alpha)*y+sin(alpha)*z,z=-sin(alpha)*y+cos(alpha)*z},f) end: 

# Symmetry z <-> y and its matrix in S-basis
symYZ := proc(f) subs({y=z,z=y},f); end:
mkMatJ := proc(l) opMatrix(symYZ, Sbasis(l)); end:

# The special permutation matrix
mkMatQ := proc(l) permutationMat([seq(2*i,i=1..l),seq(2*i+1,i=0..l)]); end:

# Invariant subspaces and symmetric subspaces of E^l by symYZ and block matrices
mkBxyz := proc(k) [seq(SS(2*k+1,-2*k-2+2*i),i=1..k)]; end:
mkExyz := proc(k) [seq(Sxyz(2*k+1,-2*k-2+2*i),i=1..k)]; end:
mkAxyz := proc(k) opMatrix(symYZ, mkExyz(k)); end:

mkBx := proc(k) [seq(SS(2*k-1,-1+2*i),i=1..k)]; end:
mkEx := proc(k) [seq(Sxyz(2*k-1,-1+2*i),i=1..k)]; end:
mkAx := proc(k) opMatrix(symYZ, mkEx(k)); end:

mkBz := proc(k) [seq(SS(2*k-1,-2+2*i),i=1..k)]; end:
mkEz := proc(k) [seq(Sxyz(2*k-1,-2+2*i),i=1..k)]; end:
mkAz := proc(k) opMatrix2(symYZ, mkEz(k), mkEy(k)); end:

mkBy := proc(k) [seq(SS(2*k-1,-2*k+2*i-1),i=1..k)]; end:
mkEy := proc(k) [seq(Sxyz(2*k-1,-2*k+2*i-1),i=1..k)]; end:
mkAy := proc(k) opMatrix2(symYZ, mkEy(k), mkEz(k)); end:

mkByz := proc(k) [seq(SS(2*k,-2*k+2*i-1),i=1..k)]; end:
mkEyz := proc(k) [seq(Sxyz(2*k,-2*k+2*i-1),i=1..k)]; end:
mkAyz := proc(k) opMatrix(symYZ, mkEyz(k)); end:

mkBxz := proc(k) [seq(SS(2*k,2*i-1),i=1..k)]; end:
mkExz := proc(k) [seq(Sxyz(2*k,2*i-1),i=1..k)]; end:
mkAxz := proc(k) opMatrix2(symYZ, mkExz(k), mkExy(k)); end:

mkBxy := proc(k) [seq(SS(2*k,-2*k+2*i-2),i=1..k)]; end:
mkExy := proc(k) [seq(Sxyz(2*k,-2*k+2*i-2),i=1..k)]; end:
mkAxy := proc(k) opMatrix2(symYZ, mkExy(k), mkExz(k)); end:

mkB1 := proc(k) [seq(SS(2*k-2,2*i-2),i=1..k)]; end:
mkE1 := proc(k) [seq(Sxyz(2*k-2,2*i-2),i=1..k)]; end:
mkA1 := proc(k) opMatrix(symYZ, mkE1(k)); end:

# Gaunt matrices
# s*E^l == E^{l+1} C_s^{l} + E^{l-1} D_s^{l} with s in {x,y,z}
mkGauntMat := proc(l,symb)
  local fam, bas, mat,m1;
  if not(symb=`x` or symb=`y` or symb=`z`) then
     ERROR("bad second argument!");
  fi;
  fam := Sbasis(l);
  bas := [op(Sbasis(l+1)), seq(f*(x^2+y^2+z^2), f=Sbasis(l-1))];
  mat := opMatrix2(f -> symb*f, fam, bas);
  m1 := submatrix(mat,1..2*l+3,1..2*l+1);
  evalm(m1);
end:

# Equivalent definition with real Gaunt coefficients
mkGauntMat2 := proc(l,symb) 
   local s,mat,i,j;
   mat := matrix(2*l+3,2*l+1,0);
   if symb=`x` then s:=1; elif symb=`y` then s:=-1; elif symb=`z` then s:=0; else
      ERROR("bad second argument!");
   fi;
   for i to 2*l+3 do
      for j to 2*l+1 do
         mat[i,j] := (-1)^s*2*sqrt(Pi)/sqrt(3)*realGaunt(l+1,l,1,i-l-2,j-l-1,s);
         mat[i,j] := simplify(combine(mat[i,j]));
      od;
   od;
   mat;
end:

# Rotation with z-axis
mkMatZ := proc(l,a)
  local mat,i;
  mat := matrix(2*l+1,2*l+1,0);
  for i to l do
     mat[i,i] := cos((l+1-i)*a);
     mat[i,2*l+2-i] := sin((l+1-i)*a);
     mat[2*l+2-i,i] := - sin((l+1-i)*a);
     mat[2*l+2-i,2*l+2-i] := cos((l+1-i)*a);
  od;
  mat[l+1,l+1]:=1;
  mat;
end:
