# rotation3.mpl
read("rotation.mpl"):
read("TABMATJ.mpl"):
read("TABMATJNUM.mpl"):

distFrobenius := proc(m1,m2)
   local res,i,j;
   res := 0.0;
   for i to rowdim(m1) do
      for j to coldim(m1) do
        res := res + (m1[i,j]-m2[i,j])^2;
      od;
   od;
   sqrt(res);
end:

# Exact rotation matrices
#########################  
MAXRND := 100:
RRR := rand(-MAXRND..MAXRND):

rndCouple := proc()
    local a,b,d;
    a := RRR();
    b := RRR();
    d:=a^2+b^2;
    if d=0 then RETURN(rndCouple()) fi;
    [(a^2-b^2)/d, 2*a*b/d];
end:

# Compute [[cos(t),cos(2*t),...,cos(n*t)],[sin(t),...,sin(n*t)]] given cpl = [cos(t),sin(t)]
listCosSin := proc(cpl,n)
  local co,si,i;
  co := cpl[1];
  si := cpl[2];
  [[seq(T(i,co),i=1..n)],[seq(si*U(i-1,co),i=1..n)]];
end:

mkRatZ := proc(cpl,l)
   local lcs,mat,i;
   lcs := listCosSin(cpl,l);
   mat := matrix(2*l+1,2*l+1,0);
   for i to l do
     mat[l+1-i,l+1-i] := lcs[1,i];
     mat[l+1+i,l+1+i] := lcs[1,i];
     mat[l+1-i,l+1+i] := lcs[2,i];
     mat[l+1+i,l+1-i] := -lcs[2,i];
   od;
   mat[l+1,l+1] := 1;
   mat;
end:

evalfMat := proc(mat) map(f->evalf(f), mat); end:

mkExactRot := proc(lcpl,l)
   local mJ;
   mJ := TABMATJ[l];
   map(f->simplify(f),
      evalm(mkRatZ(lcpl[1],l) &* mJ &* mkRatZ(lcpl[2],l) &* mJ &* mkRatZ(lcpl[3],l))
       );
end:

cpl2angle := proc(cpl)
   local t;
   Digits :=30;
   t := evalf(arccos(cpl[1]));
   if cpl[2]<0 then t:=-t; fi;
   t;
end:

# Numeric rotation matrices
###########################
mkNumerZ := proc(l,angle)
   local mat,i;
   mat := matrix(2*l+1,2*l+1,0);
   for i to l do
     mat[l+1-i,l+1-i] := cos(i*angle);
     mat[l+1+i,l+1+i] := mat[l+1-i,l+1-i];
     mat[l+1-i,l+1+i] := sin(i*angle);
     mat[l+1+i,l+1-i] := - mat[l+1-i,l+1+i];
   od;
   mat[l+1,l+1] := 1;
   mat;
end:

mkNumerRot := proc(l,alpha,beta,gamma)
   local mJ;
   Digits := 30;
    mJ := TABMATJNUM[l];
    evalm(mkNumerZ(l,alpha) &* mJ &* mkNumerZ(l,beta) &* mJ &* mkNumerZ(l,gamma));
end:      
   
mkNumerRots := proc(lmax,alpha,beta,gamma)
   local l;
   [seq(mkNumerRot(l,alpha,beta,gamma), l=2..lmax)];
end:   

# Calling the C rotation function
#################################
callCRots := proc(lmax,alpha,beta,gamma)
   local f,lmat,mat,l,i,j,ls;
   f := fopen("rot.in",WRITE);
   fprintf(f,"%d\n%30.26e %30.26e %30.26e\n",lmax,alpha,beta,gamma);
   fclose(f);
   ssystem("perl doo.pl ./test_rotation<rot.in>rot.out"); 
   f := fopen("rot.out",READ);
   lmat := [];
   Digits := 20:
   for l from 2 to lmax do
      mat[l] := matrix(2*l+1,2*l+1,0);
      for i to 2*l+1 do
        for j to 2*l+1 do
          ls := fscanf(f,"%e");
          mat[l][i,j] := ls[1];
        od;
      od;    
      lmat := [op(lmat), mat[l]];
   od;
   fclose(f);
   lmat;
end:      
   
# Comparison between C and Maple numerical results
##################################################
errorRots := proc(lmax,alpha,beta,gamma)
   local lmatc,lmatex,i;
   lmatc  := callCRots(lmax,alpha,beta,gamma);
   lmatex := mkNumerRots(lmax,alpha,beta,gamma);
   [alpha,beta,gamma, seq(distFrobenius(lmatex[i],lmatc[i]), i = 1..lmax-1)];
end:   
   
