#include "rotation.h"

////////////////////////////////////////////////////////////////////////////////
//! Read precomputed matrix data (for rotation around the Y axis)
////////////////////////////////////////////////////////////////////////////////
MatL*
read_matJ(int lmax, const char* fname)
{
  int l,k,k1,i,j,lmaxmax;
  double x;
  char com[50];
  MatL *matj;
  FILE *fin;

  if ((fin = fopen( fname,"r")) == NULL) {
    fprintf(stderr,"Cannot open input file.\n");
  }

  matj = matl_calloc(lmax);

  fscanf(fin,"%s %d",com,&lmaxmax);
  if (lmax>lmaxmax) {
    fprintf(stderr,"lmax is too small in numMatJ.dat :: %i\n", lmaxmax);
  }

#if 0
  gsl_matrix_set(matj->lmat[0],0,0, 1.0);
  gsl_matrix_set(matj->lmat[1],0,1, -1.0);
  gsl_matrix_set(matj->lmat[1],1,0, -1.0);
  gsl_matrix_set(matj->lmat[1],1,1, 1.0);
#endif

#if 1
  gsl_matrix_set(matj->lmat[0],0,0, 1.0);
  gsl_matrix_set(matj->lmat[1],0,1, -1.0);
  gsl_matrix_set(matj->lmat[1],1,0, -1.0);
  gsl_matrix_set(matj->lmat[1],2,2, 1.0);
#endif

  for (l=2;l<=lmax;l++) {
    fscanf(fin,"%s %d",com,&lmaxmax);
    k = l/2;
    k1 = k;
    for (i=0;i<k;i++) {
      for (j=0;j<i;j++) {
        fscanf(fin,"%lf",&x);
        gsl_matrix_set(matj->lmat[l],2*i+1,2*j+1,x);
        gsl_matrix_set(matj->lmat[l],2*j+1,2*i+1,x);
      }
      fscanf(fin,"%lf",&x);
      gsl_matrix_set(matj->lmat[l],2*i+1,2*i+1,x);
    }
    if (l!=2*k1) k++;
    for (i=0;i<k;i++) {
      for (j=0;j<k;j++) {
        fscanf(fin,"%lf",&x);
        gsl_matrix_set(matj->lmat[l],2*k1+2*j+1,2*i,x);
        gsl_matrix_set(matj->lmat[l],2*i,2*k1+2*j+1,x);
      }
    }
    if (l==2*k1) {k++;} else {k1++;}
    for (i=0;i<k;i++) {
      for (j=0;j<i;j++) {
        fscanf(fin,"%lf",&x);
        gsl_matrix_set(matj->lmat[l],2*k1+2*i,2*k1+2*j,x);
        gsl_matrix_set(matj->lmat[l],2*k1+2*j,2*k1+2*i,x);
      }
      fscanf(fin,"%lf",&x);
      gsl_matrix_set(matj->lmat[l],2*k1+2*i,2*k1+2*i,x);
    } 
  }
  return matj;
}

////////////////////////////////////////////////////////////////////////////////
//! 
////////////////////////////////////////////////////////////////////////////////
int   
matl_product_Zalpha_left(const MatL *matl, const double alpha, MatL *res) 
{
  int l,i,j;
  double co,si,x,y;
  extern double cos(double);
  extern double sin(double);

  for (l=1;l<=res->lmax;l++) {
    for (j=0;j<=2*l;j++) {
      gsl_matrix_set(res->lmat[l],l,j,gsl_matrix_get(matl->lmat[l],l,j));
    }
  }

  for (i=1;i<=res->lmax;i++) {

    co = cos(i*alpha);
    si = sin(i*alpha);

    for (l=i;l<=res->lmax;l++) {
      for (j=0;j<=2*l;j++) {
        x = gsl_matrix_get(matl->lmat[l],l-i,j);
        y = gsl_matrix_get(matl->lmat[l],l+i,j);
        gsl_matrix_set(res->lmat[l],l-i,j, co*x+si*y);
        gsl_matrix_set(res->lmat[l],l+i,j, -si*x+co*y);
      }	   
    }
  }
  return EXIT_SUCCESS;
}


////////////////////////////////////////////////////////////////////////////////
//! 
////////////////////////////////////////////////////////////////////////////////
int   
matl_product_Zalpha_left_n( const MatL *matl, const double alpha, MatL *res, 
                            const unsigned int max_band ) 
{
  int l,i,j;
  double co,si,x,y;
  extern double cos(double);
  extern double sin(double);

  for (l=1;l<=max_band;l++) {
    for (j=0;j<=2*l;j++) {
      gsl_matrix_set(res->lmat[l],l,j,gsl_matrix_get(matl->lmat[l],l,j));
    }
  }

  for (i=1;i<=max_band;i++) {

    co = cos(i*alpha);
    si = sin(i*alpha);

    for (l=i;l<=max_band;l++) {
      for (j=0;j<=2*l;j++) {
        x = gsl_matrix_get(matl->lmat[l],l-i,j);
        y = gsl_matrix_get(matl->lmat[l],l+i,j);
        gsl_matrix_set(res->lmat[l],l-i,j, co*x+si*y);
        gsl_matrix_set(res->lmat[l],l+i,j, -si*x+co*y);
      }	   
    }
  }
  return EXIT_SUCCESS;
}

////////////////////////////////////////////////////////////////////////////////
//! 
////////////////////////////////////////////////////////////////////////////////
int   
matl_product_Zalpha_right(const MatL *matl, const double alpha,MatL *res) 
{
  int l,i,j;
  double co,si,x,y;
  extern double cos(double);
  extern double sin(double);

  for (l=1;l<=matl->lmax;l++)
    for (i=0;i<=2*l;i++) 
      gsl_matrix_set(res->lmat[l],i,l,gsl_matrix_get(matl->lmat[l],i,l));

  for (j=1;j<=matl->lmax;j++) {
    co = cos(j*alpha);
    si = sin(j*alpha);
    for (l=j;l<=matl->lmax;l++) {
      for (i=0;i<=2*l;i++) {
        x = gsl_matrix_get(matl->lmat[l],i,l-j);
        y = gsl_matrix_get(matl->lmat[l],i,l+j);
        gsl_matrix_set(res->lmat[l],i,l-j, co*x-si*y);
        gsl_matrix_set(res->lmat[l],i,l+j, si*x+co*y);
      }	   
    }
  }
  return EXIT_SUCCESS;
}

////////////////////////////////////////////////////////////////////////////////
//! 
////////////////////////////////////////////////////////////////////////////////
int   
matl_product_Zalpha_right_n( const MatL *matl, const double alpha,MatL *res,
                             const unsigned int max_band ) 
{
  int l,i,j;
  double co,si,x,y;
  extern double cos(double);
  extern double sin(double);

  for (l=1;l<=max_band;l++)
    for (i=0;i<=2*l;i++) 
      gsl_matrix_set(res->lmat[l],i,l,gsl_matrix_get(matl->lmat[l],i,l));

  for (j=1;j<=max_band;j++) {
    co = cos(j*alpha);
    si = sin(j*alpha);
    for (l=j;l<=max_band;l++) {
      for (i=0;i<=2*l;i++) {
        x = gsl_matrix_get(matl->lmat[l],i,l-j);
        y = gsl_matrix_get(matl->lmat[l],i,l+j);
        gsl_matrix_set(res->lmat[l],i,l-j, co*x-si*y);
        gsl_matrix_set(res->lmat[l],i,l+j, si*x+co*y);
      }	   
    }
  }
  return EXIT_SUCCESS;
}

////////////////////////////////////////////////////////////////////////////////
//! Perform rotation
////////////////////////////////////////////////////////////////////////////////
int 
matl_rotation( int lmax, double alpha, double beta, double gamma, 
               MatL *res, const MatL *matj)
{
  unsigned int band, i, j;
  MatL *tmp; 

  // initialize temporary matrix as identity
  tmp = matl_calloc(lmax);

  matl_product_Zalpha_left(matj,beta,tmp);
  matl_product_symm_n(CblasLeft ,matj, tmp, res, lmax);
  matl_product_Zalpha_left(res,alpha,tmp);
  matl_product_Zalpha_right(tmp,gamma,res);

  matl_free(tmp);
  return EXIT_SUCCESS;
}

////////////////////////////////////////////////////////////////////////////////
//! Perform rotation (optimized version without allocation of temporaries)
////////////////////////////////////////////////////////////////////////////////
int 
matl_rotation_opt( int lmax, double alpha, double beta, double gamma, 
                   MatL *res, const MatL *matj, const MatL* tmp)
{
  matl_product_Zalpha_left_n(matj,beta,tmp, lmax);
  // matl_product_symm_n(CblasLeft ,matj, tmp, res, lmax);
  gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1.0,matj,tmp,0.0,res);
  matl_product_Zalpha_left_n(res,alpha,tmp, lmax);
  matl_product_Zalpha_right_n(tmp,gamma,res, lmax);

  return EXIT_SUCCESS;
}
