#include "rotation.h"
#include <time.h>

/* Returns a random number between 0 and pi */
double myrand()
{
  return( M_PI*((double) rand())/((double) RAND_MAX) );
}

static clock_t cputime;

void cpu_time_reset(void)
{
  cputime = clock();
}

double cpu_time_get(void)
{
   double dt;
   dt = (double) (clock() - cputime);
   dt = dt/CLOCKS_PER_SEC;
   return dt;
}

int main(int argc, char *argv[])
{
   int lmax, maxiter, i,j,l,iter;
   double x,y,alpha,beta,gamma;
   double tot1,tot2, tim1, tim2;
   gsl_matrix *matr,*matj;
   FILE *fres,*frand;

   lmax    = atoi(argv[1]);
   maxiter = atoi(argv[2]);

   if ((frand = fopen("rand.out","w")) == NULL) {
    fprintf(stderr,"Cannot open output file rand.out for writing!.\n");
    return EXIT_FAILURE;
   }
   if ((fres = fopen("res.out","w")) == NULL) {
    fprintf(stderr,"Cannot open output file res.out for writing!.\n");
    return EXIT_FAILURE;
   }

/* Computing individual rotation matrices J_l for l=2..lmax using read_matJ() and mat_rotation() */
   tot1 = 0.0;
   tot2 = 0.0;
   for (l=2;l<=lmax;l++) {
     /* reading precomputed entries of J_l */
     /* iteration because of the "granularity if function clock() */
     cpu_time_reset();
     matj = read_matJ(l);
     tim1 = cpu_time_get();
     tot1 += tim1;

     /* computing the rotation matrix */
     cpu_time_reset();
     for (iter=0;iter<maxiter;iter++) {
       alpha = 2*myrand();
       beta = myrand();
       gamma = 2*myrand();
       matr = gsl_matrix_calloc(2*l+1,2*l+1);
       fprintf(frand,"%10.6e %10.6e %10.6e\n",alpha,beta,gamma);

       mat_rotation(l, alpha, beta, gamma, matr, matj);

       gsl_matrix_free(matr);
     }
     tim2 = cpu_time_get();
     tim2 = tim2/maxiter;
     tot2 += tim2;

     fprintf(fres,"%d  %10.6e %10.6e %10.6e\n",l,tim1,tim2,tim1+tim2);
     printf("%d  %10.6e %10.6e %10.6e\n",l,tim1,tim2,tim1+tim2);
     
     gsl_matrix_free(matj);
     printf("l=%d\n",l);
   }
  
   printf("Total time for lmax = %d: %10.6e, %10.6e, %10.6e\n", lmax, tot1,tot2,tot1+tot2);

   fclose(frand);
   fclose(fres);
   return EXIT_SUCCESS;
}
