/// @file
////////////////////////////////////////////////////////////////////////////////
///
/// Copyright (C) 2009      University of Toronto
///
////////////////////////////////////////////////////////////////////////////////
///
///  module     : main
///
///  author     : lessig@dgp.toronto.edu
///
///  project    : Spherical Harmonics rotation
///
///  description: Project main file
///
///  license    : See README for details.
///
////////////////////////////////////////////////////////////////////////////////

// includes, files
#include <pinchon/rotation.h>

// includes, system
#include <stdio.h>

////////////////////////////////////////////////////////////////////////////////
//! Allocate matrix
////////////////////////////////////////////////////////////////////////////////
MatL*  
allocMatL( unsigned int max_band) {
  return matl_calloc( max_band);
}

////////////////////////////////////////////////////////////////////////////////
//! Free matrix
////////////////////////////////////////////////////////////////////////////////
void 
freeMatL( MatL* mat) {
  matl_free( mat);
}

////////////////////////////////////////////////////////////////////////////////
//! Allocate vector
////////////////////////////////////////////////////////////////////////////////
VecL*  
allocVecL( unsigned int max_band) {
  return vecl_calloc( max_band);
}

////////////////////////////////////////////////////////////////////////////////
//! Free vector
////////////////////////////////////////////////////////////////////////////////
void 
freeVecL( VecL* mat) {
  vecl_free( mat);
}


////////////////////////////////////////////////////////////////////////////////
// Read precomputed matrix data (for rotation around the Y axis)
////////////////////////////////////////////////////////////////////////////////
void 
pinchonPreloadJ( const unsigned int max_band, MatL** matJ) {
  *matJ = read_lmatJ(max_band);
}

////////////////////////////////////////////////////////////////////////////////
//! Compute the rotation matrix for ZYZ rotation angles
////////////////////////////////////////////////////////////////////////////////
void
pinchonRotationMatrix( const unsigned int max_band, double alpha, double beta,
                       double gamma, const MatL* mat_j, MatL* mat_r ) 
{
  // compute rotation matrix
  // matl_rotation_opt( max_band, alpha, beta, gamma, mat_r, mat_j, mat_temp);
  matl_rotation( max_band, alpha, beta, gamma, mat_r, mat_j);
}

////////////////////////////////////////////////////////////////////////////////
//! Compute the rotation of SH coefficients
////////////////////////////////////////////////////////////////////////////////
void
pinchonRotateVector( const unsigned int max_band, double alpha, double beta,
                     double gamma, VecL* sh_vec, VecL* sh_vec_rot,
                     const MatL* mat_j) 
{
  vecl_rotation( max_band, alpha, beta, gamma, sh_vec, sh_vec_rot, mat_j);
}

